from typing import Optional

from pydantic import BaseModel, Field

from .request_filter import Filter
from .request_optional_parameters import OptionalParameters
from .request_part import Part


class YouTubeRequest(BaseModel):
    part: Part
    filter: Optional[Filter] = Filter()
    optional_parameters: Optional[OptionalParameters] = OptionalParameters()



from .request_filter import Filter
from .request_optional_parameters import OptionalParameters
from .request_part import Part
from .video import VideoFilter, VideoOptionalParameters, VideoPart
from .youtube_request import YouTubeRequest



from pydantic import BaseModel, Field


class ContentRating(BaseModel):
    # acb: str = ''
    # agcom: str = ''
    # anatel: str = ''
    # bbfc: str = ''
    # bfvc: str = ''
    # bmukk: str = ''
    # catv: str = ''
    # catvfr: str = ''
    # cbfc: str = ''
    # ccc: str = ''
    # cce: str = ''
    # chfilm: str = ''
    # chvrs: str = ''
    # cicf: str = ''
    # cna: str = ''
    # cnc: str = ''
    # csa: str = ''
    # csc: str = ''
    # czfilm: str = ''
    # djctq: str = ''
    # ecbm: str = ''
    # eefilm: str = ''
    # egfilm: str = ''
    # eirin: str = ''
    # fcbm: str = ''
    # fco: str = ''
    # fmoc: str = ''
    # fpb: str = ''
    # fpbfl: str = ''
    # fsk: str = ''
    # grfilm: str = ''
    # icaa: str = ''
    # ifco: str = ''
    # ilfilm: str = ''
    # incaa: str = ''
    # kfcb: str = ''
    # kijkwijzer: str = ''
    # kmrb: str = ''
    # lsf: str = ''
    # mccaa: str = ''
    # mccyp: str = ''
    # mcst: str = ''
    # mda: str = ''
    # medietilsynet: str = ''
    # meku: str = ''
    # mf: str = ''
    # mibac: str = ''
    # moc: str = ''
    # moctw: str = ''
    # mpaa: str = ''
    # mpaat: str = ''
    # mtrcb: str = ''
    # nbc: str = ''
    # nbcpl: str = ''
    # nfrc: str = ''
    # nfvcb: str = ''
    # nkclv: str = ''
    # oflc: str = ''
    # pefilm: str = ''
    # rcnof: str = ''
    # resorteviolencia: str = ''
    # rtc: str = ''
    # rte: str = ''
    # russia: str = ''
    # skfilm: str = ''
    # smais: str = ''
    # smsa: str = ''
    # tvpg: str = ''
    # yt: str = ''
    pass



from typing import Any

from ...models import (
    FileDetails,
    LiveStreamingDetails,
    Localizations,
    Localized,
    PaidProductPlacementDetails,
    Player,
    ProcessingDetails,
    RecordingDetails,
    Snippet,
    Statistics,
    Status,
    Suggestions,
    Thumbnail,
    TopicDetails,
    Video,
)


def parse_thumbnail(resolution: str, details: dict) -> Thumbnail:
    thumbnail: Thumbnail = Thumbnail()
    thumbnail.resolution = resolution
    if details.get("url"):
        thumbnail.url = details["url"]
    if details.get("width"):
        thumbnail.width = details["width"]
    if details.get("height"):
        thumbnail.height = details["height"]
    return thumbnail


def parse_localized(localized_data: dict[str, Any]) -> Localized:
    pass


def parse_snippet(snippet_data: dict[str, Any]) -> Snippet:
    snippet: Snippet = Snippet()
    if snippet_data.get("title"):
        snippet.title = snippet_data["title"]
    if snippet_data.get("description"):
        snippet.description = snippet_data["description"]
    if snippet_data.get("thumbnails"):
        snippet.thumbnails = [
            parse_thumbnail(resolution, details)
            for resolution, details in snippet_data["thumbnails"].items()
        ]
    if snippet_data.get("publishedAt"):
        snippet.publishedAt = snippet_data["publishedAt"]
    if snippet_data.get("channelId"):
        snippet.channelId = snippet_data["channelId"]
    if snippet_data.get("channelTitle"):
        snippet.channelTitle = snippet_data["channelTitle"]
    if snippet_data.get("categoryId"):
        snippet.categoryId = snippet_data["categoryId"]
    if snippet_data.get("localized"):
        snippet.localized = parse_localized(snippet_data["localized"])
    if snippet_data.get("defaultAudioLanguage"):
        snippet.defaultAudioLanguage = snippet_data["defaultAudioLanguage"]
    if snippet_data.get("defaultLanguage"):
        snippet.defaultLanguage = snippet_data["defaultLanguage"]
    if snippet_data.get("tags"):
        snippet.tags = snippet_data["tags"]
    return snippet


def parse_status(status_data: dict[str, int]) -> Status:
    pass


def parse_statistics(statistics_data: dict[str, int]) -> Statistics:
    statistics: Statistics = Statistics()
    if statistics_data.get("viewCount"):
        statistics.viewCount = statistics_data["viewCount"]
    if statistics_data.get("likeCount"):
        statistics.likeCount = statistics_data["likeCount"]
    if statistics_data.get("dislikeCount"):
        statistics.dislikeCount = statistics_data["dislikeCount"]
    if statistics_data.get("favoriteCount"):
        statistics.favoriteCount = statistics_data["favoriteCount"]
    if statistics_data.get("commentCount"):
        statistics.commentCount = statistics_data["commentCount"]
    return statistics


def parse_paid_product_details(
    statistics_data: dict[str, int],
) -> PaidProductPlacementDetails:
    pass


def parse_player(statistics_data: dict[str, int]) -> Player:
    pass


def parse_topic_details(topic_details_data: dict[str, int]) -> TopicDetails:
    pass


def parse_recording_details(recording_data: dict[str, int]) -> RecordingDetails:
    pass


def parse_file_details(file_data: dict[str, int]) -> FileDetails:
    pass


def parse_processing_details(processing_data: dict[str, int]) -> ProcessingDetails:
    pass


def parse_suggestions(suggestions_data: dict[str, int]) -> Suggestions:
    pass


def parse_livestreaming_details(
    livestreaming_data: dict[str, int],
) -> LiveStreamingDetails:
    pass


def parse_localizations(localizations_data: dict[str, int]) -> Localizations:
    pass


class VideoParser:
    def __init__(self, video_result: dict, video: Video = Video()):
        self.video_result = video_result
        self.video = video

    def parse_kind(self) -> Video:
        if "kind" in self.video_result:
            self.video.kind = self.video_result["kind"]
        return self.video

    def parse_etag(self) -> Video:
        if "etag" in self.video_result:
            self.video.etag = self.video_result["etag"]
        return self.video

    def parse_id(self) -> Video:
        if "id" in self.video_result:
            self.video.id = self.video_result["id"]
        return self.video

    def parse_snippet(self) -> Video:
        if "snippet" in self.video_result:
            self.video.snippet = parse_snippet(self.video_result["snippet"])
        return self.video

    def parse_status(self) -> Video:
        if "status" in self.video_result:
            self.video.status = parse_status(self.video_result["status"])
        return self.video

    def parse_statistics(self) -> Video:
        if "statistics" in self.video_result:
            self.video.statistics = parse_statistics(self.video_result["statistics"])
        return self.video

    def parse_paid_product_placement_details(self) -> Video:
        if "paidProductPlacementDetails" in self.video_result:
            self.video.paidProductPlacementDetails = parse_paid_product_details(
                self.video_result["paidProductPlacementDetails"]
            )
        return self.video

    def parse_player(self) -> Video:
        if "player" in self.video_result:
            self.video.player = parse_player(self.video_result["player"])
        return self.video

    def parse_topic_details(self) -> Video:
        if "topicDetails" in self.video_result:
            self.video.topicDetails = parse_topic_details(
                self.video_result["topicDetails"]
            )
        return self.video

    def parse_recording_details(self) -> Video:
        if "recordingDetails" in self.video_result:
            self.video.recordingDetails = parse_recording_details(
                self.video_result["recordingDetails"]
            )
        return self.video

    def parse_file_details(self) -> Video:
        if "fileDetails" in self.video_result:
            self.video.fileDetails = parse_file_details(
                self.video_result["fileDetails"]
            )
        return self.video

    def parse_processing_details(self) -> Video:
        if "processingDetails" in self.video_result:
            self.video.processingDetails = parse_processing_details(
                self.video_result["processingDetails"]
            )
        return self.video

    def parse_suggestions(self) -> Video:
        if "suggestions" in self.video_result:
            self.video.suggestions = parse_suggestions(self.video_result["suggestions"])
        return self.video


def parse_video(video_result: dict) -> Video:
    video = Video()
    video_parser = VideoParser(video_result, video)
    video_parser.parse_snippet()
    video_parser.parse_status()
    video_parser.parse_statistics()
    video_parser.parse_paid_product_placement_details()
    video_parser.parse_player()
    video_parser.parse_topic_details()
    video_parser.parse_recording_details()
    video_parser.parse_file_details()
    video_parser.parse_processing_details()
    video_parser.parse_suggestions()
    return video


def parse_video_list(video_result: list[dict]) -> list[Video]:
    videos: list[Video] = list(map(parse_video, video_result))
    return videos



from typing import Optional

from pydantic import BaseModel, Field

from .page_info import PageInfo
from .video import Video


class VideoListResponse(BaseModel):
    kind: str
    etag: str
    nextPageToken: str = ""
    prevPageToken: str = ""
    pageInfo: Optional[PageInfo] = None
    items: list[Video] = Field(default_factory=list)



from typing import Any

from ...exceptions import VideoNotFoundException
from ...models import Video
from ...schemas import VideoFilter, VideoOptionalParameters, VideoPart, YouTubeRequest
from ..utils import create_request_dict
from .parsers import parse_video, parse_video_list


class YouTubeVideoResource:
    def __init__(self, youtube_client: Any):
        self.youtube_client = youtube_client

    def find_video_by_id(self, video_id: str) -> Video:
        request_filter: VideoFilter = VideoFilter(id=[video_id])
        request_part: VideoPart = VideoPart()
        request_optional_params: VideoOptionalParameters = VideoOptionalParameters()
        youtube_request: YouTubeRequest = YouTubeRequest(
            part=request_part,
            filter=request_filter,
            optional_parameters=request_optional_params,
        )
        request_dict = create_request_dict(youtube_request)
        find_video_request: dict = self.youtube_client.videos().list(**request_dict)
        find_video_result: dict[str, Any] = find_video_request.execute()
        if not find_video_result["items"]:
            raise VideoNotFoundException("No such video exists")
        video: Video = parse_video(find_video_result["items"][0])
        return video

    def find_videos_by_ids(self, video_ids: str) -> list[Video]:
        request_filter: VideoFilter = VideoFilter(id=video_ids)
        request_part: VideoPart = VideoPart()
        request_optional_params: VideoOptionalParameters = VideoOptionalParameters()
        youtube_request: YouTubeRequest = YouTubeRequest(
            part=request_part,
            filter=request_filter,
            optional_parameters=request_optional_params,
        )
        request_dict = create_request_dict(youtube_request)
        find_video_request: dict = self.youtube_client.videos().list(**request_dict)
        find_video_result: dict[str, Any] = find_video_request.execute()
        if not find_video_result["items"]:
            raise VideoNotFoundException("No such videos exist")
        videos = parse_video_list(find_video_result["items"])
        return videos



from .file_details import FileDetails
from .livestreaming_details import LiveStreamingDetails
from .localizations import Localizations
from .localized import Localized
from .page_info import PageInfo
from .paid_product_placement_details import PaidProductPlacementDetails
from .player import Player
from .processing_details import ProcessingDetails
from .recording_details import RecordingDetails
from .resource import Resource
from .response import Response
from .snippet import Snippet
from .statistics import Statistics
from .status import Status
from .suggestions import Suggestions
from .thumbnail import Thumbnail
from .topic_details import TopicDetails
from .video import Video
from .video_list_response import VideoListResponse



from typing import Literal

from pydantic import BaseModel

from .content_rating import ContentRating
from .region_restriction import RegionRestriction
from .status import Status


class ContentDetails(BaseModel):
    duration: str
    dimension: str
    definition: Literal["hd", "sd"]
    definition: Literal["false", "true"]
    licensedContent: bool
    regionRestriction: RegionRestriction
    duration: str
    contentRating: ContentRating
    projection: Literal["360", "rectangular"]
    status: Status



from typing import Optional

from pydantic import BaseModel, Field


class Statistics(BaseModel):
    viewCount: Optional[int] = Field(default=None)
    likeCount: Optional[int] = Field(default=None)
    dislikeCount: Optional[int] = Field(default=None)
    favoriteCount: Optional[int] = Field(default=None)
    commentCount: Optional[int] = Field(default=None)



from typing import Any

from ..schemas import Filter, OptionalParameters, Part, YouTubeRequest


def generate_part(part: Part) -> str:
    """Generate the 'Part' for a request.

    Parameters
    ----------
    part: Part
        An instance of part that contains all the required details.
    """
    part_str: str = ",".join(part.part)
    return part_str


def generate_optional_parameters(optional_params: OptionalParameters) -> dict[str, Any]:
    """Generate Optional parmateers for a request.

    Parameters
    ----------
    optional_parameters: OptionalParameters
        Optional parameters used when sending the request
    """
    optional: dict[str, Any] = dict()
    for key, value in optional_params.model_dump().items():
        #  We only want optional parameters that were provided
        if value:
            optional[key] = value
    #  This is useful when sending search request and type is provided as a list i.e ['video']
    if optional_params.model_dump().get("type"):
        optional["type"] = ",".join(optional_params.type)
    return optional


def generate_filter(request_filter: Filter) -> dict[str, Any]:
    """Generate the filters for request to youtube."""
    #  We pick the very first filter.
    for key, value in request_filter.model_dump().items():
        if value:
            if key == "id":
                value: str = ",".join(value)
            return {key: value}
    return {}


def create_request_dict(request_schema: YouTubeRequest) -> dict[str, Any]:
    """Create the request dict for sending request to youtube."""
    request_dict: dict[str, Any] = dict()
    request_dict["part"] = generate_part(request_schema.part)
    request_dict.update(
        generate_optional_parameters(request_schema.optional_parameters)
    )
    request_dict.update(generate_filter(request_schema.filter))
    return request_dict
